<?php
/**
 * Class AdminController.
 *
 * @package ILabs\WoocommerceDHL24
 */

namespace ILabs\WoocommerceDHL24\Controller;

use ILabs\WoocommerceDHL24\Helper\Param;
use ILabs\WoocommerceDHL24\Helper\Notice;
use ILabs\WoocommerceDHL24\Helper\Pieces;
use ILabs\WoocommerceDHL24\PluginOptions;
use ILabs\WoocommerceDHL24\Helper\Defaults;
use ILabs\WoocommerceDHL24\Helper\Services;
use ILabs\WoocommerceDHL24\Helper\Shipment;
use ILabs\WoocommerceDHL24\Helper\AdminAjax;
use ILabs\WoocommerceDHL24\Helper\Comparator;
use ILabs\WoocommerceDHL24\Helper\Normalizer;
use ILabs\WoocommerceDHL24\Exception\ExceptionInterface;
use ILabs\WoocommerceDHL24\Repository\ShipmentsRepository;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\LabelsDatas;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\ShipmentBasicDatas;
use ILabs\WoocommerceDHL24\Helper\Template;

/**
 * AdminController
 */
class AdminController extends AbstractController implements ControllerInterface {

	/**
	 * Normalizer
	 *
	 * @var Normalizer
	 */
	private $normalizer;

	/**
	 * Comparator
	 *
	 * @var Comparator
	 */
	private $comparator;

	/**
	 * Construct
	 *
	 * @param PluginOptions $plugin_options Plugin options.
	 * @return void
	 */
	public function __construct( PluginOptions $plugin_options ) {
		$this->plugin_options = $plugin_options;
	}

	/**
	 * Init class.
	 */
	public function init() {
		add_action( 'admin_menu', array( $this, 'admin_menu' ) );
		add_action( 'admin_enqueue_scripts', array( $this, 'admin_enqueue_scripts' ) );
		add_action( 'wp_ajax_dhl24_get_service', array( $this, 'ajax_get_service' ) );
		add_action( 'wp_ajax_dhl24_order_courier', array( $this, 'ajax_order_courier' ) );
		add_action( 'wp_ajax_dhl24_new_shipment', array( $this, 'ajax_new_shipment' ) );
		add_action( 'wp_ajax_dhl24_return_shipment', array( $this, 'ajax_return_shipment' ) );
		add_action( 'wp_ajax_dhl24_shipment_confirmation', array( $this, 'ajax_shipment_confirmation' ) );
		add_action( 'wp_ajax_dhl24_template', array( $this, 'ajax_template' ) );
		add_action( 'wp_ajax_dhl24_template_delete', array( $this, 'ajax_template_delete' ) );

		$this->normalizer = new Normalizer();
		$this->comparator = new Comparator();
	}

	/**
	 * Reads information from the request and renders the content of a page
	 *
	 * @return void
	 */
	public function do_action() {
		if ( ! is_admin() && ! current_user_can( 'manage_options' ) ) {
			return;
		}
        // phpcs:ignore WordPress.Security.NonceVerification
		$path = ! empty( $_GET['path'] ) ? sanitize_text_field( wp_unslash( $_GET['path'] ) ) : '';

		if ( 1 === preg_match( '/^\/(?P<action>[a-z_]+)$/', $path, $matches ) ) {
			$action_method = 'action_' . $matches['action'];
			if ( method_exists( $this, $action_method ) ) {
				$this->$action_method();

				return;
			}
		}
		$this->action_my_parcels();

		// $this->render( 'home.php' );
	}

	/**
	 * Register pages
	 *
	 * @return void
	 */
	public function admin_menu() {
		add_menu_page(
			__( 'DHL24', 'woocommerce-dhl24' ),
			__( 'DHL24', 'woocommerce-dhl24' ),
			'manage_options',
			Defaults::ADMIN_PAGE,
			'',
			Param::get_instance()->get_value( 'icon' )
		);
		add_submenu_page(
			Defaults::ADMIN_PAGE,
			__( 'My parcels', 'woocommerce-dhl24' ),
			__( 'My parcels', 'woocommerce-dhl24' ),
			'manage_options',
			Defaults::ADMIN_PAGE,
			array( $this, 'do_action' )
		);
		add_submenu_page(
			Defaults::ADMIN_PAGE,
			__( 'Parcels shipment confirmation', 'woocommerce-dhl24' ),
			__( 'Parcels shipment confirmation', 'woocommerce-dhl24' ),
			'manage_options',
			Defaults::ADMIN_PAGE . '&path=/shipment_confirmation',
			array( $this, 'do_action' )
		);
		add_submenu_page(
			Defaults::ADMIN_PAGE,
			__( 'Templates', 'woocommerce-dhl24' ),
			__( 'Templates', 'woocommerce-dhl24' ),
			'manage_options',
			Defaults::ADMIN_PAGE . '&path=/templates',
			array( $this, 'do_action' )
		);
		add_submenu_page(
			Defaults::ADMIN_PAGE,
			__( 'Settings', 'woocommerce-dhl24' ),
			__( 'Settings', 'woocommerce-dhl24' ),
			'manage_options',
			admin_url( 'admin.php?page=wc-settings&tab=' . Defaults::ADMIN_PAGE )
		);
		add_submenu_page(
			Defaults::ADMIN_PAGE,
			__( 'Help', 'woocommerce-dhl24' ),
			__( 'Help', 'woocommerce-dhl24' ),
			'manage_options',
			Defaults::ADMIN_PAGE . '&path=/help',
			array( $this, 'do_action' )
		);
	}

	/**
	 * Admin enqueue scripts
	 *
	 * @internal Add action hook: {@link init()}
	 * @return void
	 */
	public function admin_enqueue_scripts() {
		if ( ! is_admin() ) {
			return;
		}
		$current_screen = get_current_screen();

		if ( 'toplevel_page_dhl24' === $current_screen->base ) {
			wp_enqueue_style(
				'datepicker',
				$this->plugin_options->get_plugin_url() . '/assets/css/datepicker.css',
				array(),
				$this->plugin_options->get_plugin_version()
			);
			wp_enqueue_style(
				Defaults::ADMIN_PAGE,
				$this->plugin_options->get_plugin_url() . '/assets/css/dhl24.css',
				array(),
				bin2hex( random_bytes( 16 ) ) // $this->plugin_options->get_plugin_version()
			);
			wp_enqueue_script( 'jquery-ui-datepicker' );
			wp_enqueue_style( 'wp-jquery-ui-dialog' );
			wp_enqueue_script( 'jquery-ui-dialog' );
			wp_enqueue_script(
				Defaults::ADMIN_PAGE,
				$this->plugin_options->get_plugin_url() . '/assets/js/dhl24.js',
				array(),
				bin2hex( random_bytes( 16 ) ), // $this->plugin_options->get_plugin_version()
				true
			);
			wp_localize_script(
				Defaults::ADMIN_PAGE,
				'dhl24_api',
				array(
					'_nonce'         => wp_create_nonce( 'dhl24_api_nonce' ),
					'ajax_url'       => esc_url( admin_url( 'admin-ajax.php' ) ),
					'shipment_url'   => esc_url( admin_url( 'admin.php?page=' . Defaults::ADMIN_PAGE . '&path=/shipment&id=' ) ),
					'my_parcels_url' => esc_url( admin_url( 'admin.php?page=' . Defaults::ADMIN_PAGE . '&path=/my_parcels' ) ),
					'order_edit_url' => esc_url( admin_url( 'post.php?action=edit&post=' ) ),
				)
			);
		}
	}

	/**
	 * Ajax get service
	 *
	 * @return void
	 */
	public function ajax_get_service() {
		if ( ! empty( $_POST['_nonce'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['_nonce'] ) ), 'dhl24_api_nonce' ) ) {
            $pickup_day   = !empty($_POST['date']) ? $this->normalizer->date($_POST['date']) : null; // phpcs:ignore
			$post_code    = ! empty( $_POST['post_code'] ) ? sanitize_text_field( wp_unslash( $_POST['post_code'] ) ) : '';
			$service_type = ! empty( $_POST['service_type'] ) ? sanitize_text_field( wp_unslash( $_POST['service_type'] ) ) : '';
			$is_dr_pickup = ! empty( $_POST['dr_pickup'] ) ? $this->normalizer->bool( sanitize_text_field( wp_unslash( $_POST['dr_pickup'] ) ) ) : false;
			if ( null !== $pickup_day && ! empty( $post_code ) && ! empty( $service_type ) ) {
				$services        = new Services();
				$available_hours = $services->get_available_hours( $post_code, $pickup_day, $is_dr_pickup );
				wp_send_json( $available_hours['hours'], \WP_Http::OK );
			}
		}
		wp_send_json( '', \WP_Http::BAD_REQUEST );
	}

	/**
	 * Process ajax order_courier
	 *
	 * @return void
	 */
	public function ajax_order_courier() {
        if (!empty($_POST['order_courier'])) { // phpcs:ignore
			$admin_ajax = new AdminAjax();
			if ( true === $admin_ajax->order_courier( wp_unslash( $_POST['order_courier'] ) ) ) { // phpcs:ignore
				return;
			}
		}

		wp_send_json( '', \WP_Http::BAD_REQUEST );
	}

	/**
	 * Process ajax new_shipment
	 *
	 * @return void
	 */
	public function ajax_new_shipment() {
        if (!empty($_POST['new_shipment'])) { // phpcs:ignore
			$admin_ajax = new AdminAjax();
			if ( true === $admin_ajax->new_shipment( wp_unslash( $_POST['new_shipment'] ) ) ) { // phpcs:ignore
				return;
			}
		}

		wp_send_json( '', \WP_Http::BAD_REQUEST );
	}

	/**
	 * Process ajax return_shipment
	 *
	 * @return void
	 */
	public function ajax_return_shipment() {
        if (!empty($_POST['return_shipment'])) { // phpcs:ignore
			$admin_ajax = new AdminAjax();
			if ( true === $admin_ajax->return_shipment( wp_unslash( $_POST['return_shipment'] ) ) ) { // phpcs:ignore
				return;
			}
		}

		wp_send_json( '', \WP_Http::BAD_REQUEST );
	}

	/**
	 * Process ajax shipment_confirmation
	 *
	 * @return void
	 */
	public function ajax_shipment_confirmation() {
        if (!empty($_POST['shipment_confirmation'])) { // phpcs:ignore
			$admin_ajax = new AdminAjax();
			if ( true === $admin_ajax->shipment_confirmation( wp_unslash( $_POST['shipment_confirmation'] ) ) ) { // phpcs:ignore
				return;
			}
		}

		wp_send_json( '', \WP_Http::BAD_REQUEST );
	}

	/**
	 * Process ajax template
	 *
	 * @return void
	 */
	public function ajax_template() {
        if (!empty($_POST['template'])) { // phpcs:ignore
			$admin_ajax = new AdminAjax();
			if ( true === $admin_ajax->shipment_template( wp_unslash( $_POST['template'] ) ) ) { // phpcs:ignore
				return;
			}
		}

		wp_send_json( '', \WP_Http::BAD_REQUEST );
	}

	/**
	 * Process ajax template delete
	 *
	 * @return void
	 */
	public function ajax_template_delete() {
        if (!empty($_POST['template_delete'])) { // phpcs:ignore
			$admin_ajax = new AdminAjax();
			if ( true === $admin_ajax->shipment_template_delete( wp_unslash( $_POST['template_delete'] ) ) ) { // phpcs:ignore
				return;
			}
		}

		wp_send_json( '', \WP_Http::BAD_REQUEST );
	}

	/**
	 * Render HTML
	 *
	 * @param string $template_name Template name.
	 * @param array  $args Additional arguments passed to the template.
	 * @return void
	 */
	protected function render( string $template_name, array $args = array() ) {
		$template_name = 'admin/' . $template_name;

		parent::render( $template_name, $args );
	}

	/**
	 * My parcels page.
	 *
	 * @return void
	 */
	private function action_my_parcels() {
		$shipments_repository = new ShipmentsRepository();

		$from = null;
		$to   = null;
		if ( ! empty( $_GET['_wpnonce'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_GET['_wpnonce'] ) ), 'my_parcels_filter' ) ) {
			$action = ! empty( $_GET['action'] ) ? sanitize_text_field( wp_unslash( $_GET['action'] ) ) : '';
			if ( '' !== $action ) {
				$this->do_bulk_action( $action );
			}
			$from = $this->parse_date( 'from' );
			if ( null !== $from ) {
				$shipments_repository->set_from( $from );
			}
			$to = $this->parse_date( 'to' );
			if ( null !== $to ) {
				$shipments_repository->set_to( $to );
			}
		}
		$paged = ! empty( $_GET['paged'] ) ? $this->normalizer->integer( sanitize_text_field( wp_unslash( $_GET['paged'] ) ) ) : 0;
		if ( 1 > $paged ) {
			$paged = 1;
		}

		$my_shipments_count = 0;
		try {
			$my_shipments_count = $shipments_repository->fetch_my_shipments_count();
		} catch ( ExceptionInterface $e ) {
			Notice::get_instance()->add( 'error', $e->getMessage() );
		}
		try {
			$my_shipments = $shipments_repository->fetch_my_shipments( $paged - 1 );
		} catch ( ExceptionInterface $e ) {
			Notice::get_instance()->add( 'error', $e->getMessage() );
			$my_shipments = new ShipmentBasicDatas();
		}
		$shipment_id_list = array();
		foreach ( $my_shipments->getShipments() as $item ) {
			$shipment_id_list[] = $item->getShipmentId();
		}
		try {
			$labels = $shipments_repository->fetch_labels( $shipment_id_list );
		} catch ( ExceptionInterface $e ) {
			Notice::get_instance()->add( 'error', $e->getMessage() );
			$labels = new LabelsDatas();
		}

		$this->render(
			'my-parcels.php',
			array(
				'my_shipments'       => $my_shipments,
				'my_shipments_count' => $my_shipments_count,
				'my_shipments_show'  => count( $my_shipments->getShipments() ),
				'labels'             => $labels,
				'from'               => $from ? $from->format( 'Y-m-d' ) : '',
				'to'                 => $to ? $to->format( 'Y-m-d' ) : '',
				'paged'              => $paged,
			)
		);
	}

	/**
	 * Templates page
	 *
	 * @return void
	 */
	private function action_templates() {
		$page_args = array(
			'templates' => Template::get_instance()->get_templates( true ),
			'_nonce'    => wp_create_nonce( 'ajax_shipment_template' ),
		);
		$this->render( 'templates.php', $page_args );
	}

	/**
	 * Help page
	 *
	 * @return void
	 */
	private function action_help() {
		$page_args = array();
		$this->render( 'help.php', $page_args );
	}

	/**
	 * Shipment page
	 *
	 * @return void
	 */
	private function action_shipment() {
		$shipment_id = $this->get_id( false );
		$shipment    = new Shipment();
		$page_args   = $shipment->get_show_page_args( $shipment_id );
		if ( empty( $page_args ) ) {
			/* translators: %s: Shipment ID */
			$this->back_to_my_parcels( sprintf( __( 'Shipment %s does not exist.', 'woocommerce-dhl24' ), $shipment_id ) );
			return;
		}

		$this->render( 'shipment.php', $page_args );
	}

	/**
	 * Action order courier
	 *
	 * @return void
	 */
	private function action_order_courier() {
		$ids = ! empty( $_GET['ids'] ) ? sanitize_text_field( wp_unslash( $_GET['ids'] ) ) : ''; // phpcs:ignore WordPress.Security.NonceVerification
		if ( '' === $ids ) {
			$this->back_to_my_parcels( __( 'The param ids is empty.', 'woocommerce-dhl24' ) );
			return;
		}
		$ids = explode( ',', $ids );

		$shipments_repository = new ShipmentsRepository();

		try {
			$labels = $shipments_repository->fetch_labels( $ids );
		} catch ( ExceptionInterface $e ) {
			Notice::get_instance()->add( 'error', $e->getMessage() );
			$labels = new LabelsDatas();
		}

		$pieces = new Pieces();
		$sender = null;

		foreach ( $labels->getItems() as $item ) {
			if ( ! empty( $item->getDispatchNotificationNumber() ) ) {
				$this->back_to_my_parcels( __( 'Courier delivery already ordered.', 'woocommerce-dhl24' ) );
				return;
			}
			if ( false === $this->comparator->is_the_same_shipper( $item->getShipper() ) ) {
				$this->back_to_my_parcels( __( 'Ordering courier to pick up several shipments is admissible provided that the same place of posting was entered for all the shipments. Make sure if it was entered so and try again.', 'woocommerce-dhl24' ) );
				return;
			}

			if ( false === $this->comparator->is_the_same_posting_date( $item->getShipmentTime() ) ) {
				$this->back_to_my_parcels( __( 'Courier delivery must not be ordered for several shipments with different posting dates.', 'woocommerce-dhl24' ) );
				return;
			}

			if ( false === $this->comparator->is_courier_ordered_individually( $item->getService() ) ) {
				$this->back_to_my_parcels( __( 'Unfortunately, one courier must not be ordered to pick up several shipments type "third party". Courier delivery must be ordered for each shipment individually.', 'woocommerce-dhl24' ) );
				return;
			}

			$pieces->add_piece_list( $item->getPieceList() );

			$sender       = $item->getShipper();
			$service_type = $item->getService()->getProduct();
		}

		$services        = new Services();
		$is_dr_pickup    = $pieces->is_dr_pickup();
		$pickup_days     = $services->get_pickup_days( $sender->getPostalCode(), $is_dr_pickup, 'yes', $service_type );
		$available_hours = $services->get_available_hours( $sender->getPostalCode(), $pickup_days[0], $pieces->is_dr_pickup() );

		wp_localize_script(
			Defaults::ADMIN_PAGE,
			'dhl24_oc',
			array(
				'post_code'    => $sender->getPostalCode(),
				'service_type' => $service_type,
			)
		);

		$this->render(
			'order-courier.php',
			array(
				'_nonce'          => wp_create_nonce( 'action_order_courier' ),
				'ids'             => $ids,
				'sender'          => $sender,
				'pickup_days'     => $pickup_days,
				'available_hours' => $available_hours,
				'piece_totals'    => $pieces->get_totals(),
				'is_dr_pickup'    => $is_dr_pickup,
			)
		);
	}

	/**
	 * Action create new shipment
	 *
	 * @return void
	 */
	private function action_new_shipment() {
		$order_id = $this->get_id();
		$shipment = new Shipment();

		$this->render( 'new-shipment.php', $shipment->get_new_page_args( $order_id ) );
	}

	/**
	 * Parcels shipment confirmation page
	 *
	 * @return void
	 */
	private function action_shipment_confirmation() {
		$today = new \DateTime();
		$this->render(
			'shipment-confirmation.php',
			array(
				'_nonce' => wp_create_nonce( 'ajax_shipment_confirmation' ),
				'date'   => $today->format( 'Y-m-d' ),
				'types'  => Param::get_instance()->get_value( 'shipment_confirmation_types' ),
			)
		);
	}

	/**
	 * Cancel courier booking
	 *
	 * @return void
	 */
	private function action_cancel_courier_booking() {
		$shipment = new Shipment();
		$shipment->cancel_courier_booking( $this->get_id( false ) );

		$this->action_my_parcels();
	}

	/**
	 * Delete shipment
	 *
	 * @return void
	 */
	private function action_delete_shipment() {
		$shipment = new Shipment();
		$shipment->delete( $this->get_id( false ) );

		$this->action_my_parcels();
	}

	/**
	 * Return shipment
	 *
	 * @return void
	 */
	private function action_return_shipment() {
		$shipment = new Shipment();

		$return_page_args = $shipment->get_return_page_args( $this->get_id( false ) );
		if ( ! isset( $return_page_args['order_id'] ) ) {
			$this->render( 'return-shipment-whithout-order.php', $return_page_args );

			return;
		}

		$this->render( 'return-shipment.php', $return_page_args );
	}

	/**
	 * Get ID from request
	 *
	 * @param  bool $cast_to_int Cast value to int.
	 * @return string|int
	 */
	private function get_id( $cast_to_int = true ) {
        // phpcs:ignore WordPress.Security.NonceVerification
		$id = ! empty( $_GET['id'] ) ? sanitize_text_field( wp_unslash( $_GET['id'] ) ) : '';

		if ( true === $cast_to_int ) {
			return (int) $id;
		}

		return $id;
	}

	/**
	 * Parse date
	 *
	 * @param  string $value_name Value name.
	 * @return \DateTime|null
	 */
	private function parse_date( string $value_name ) {
		$datetime = ! empty( $_GET[ $value_name ] ) ? wp_unslash( $_GET[ $value_name ] ) : ''; // phpcs:ignore WordPress.Security.NonceVerification, WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		return $this->normalizer->date( $datetime );
	}

	/**
	 * Do bulk action
	 *
	 * @param  string $action_name Action name.
	 * @return void
	 */
	private function do_bulk_action( string $action_name ) {
		$ids = array();
		if ( empty( $_GET['ids'] ) || ! is_array( $_GET['ids'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification
			return;
		}
		$ids = wp_unslash( $_GET['ids'] ); // phpcs:ignore WordPress.Security.NonceVerification, WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		$ids = array_map( 'sanitize_text_field', $ids );
		if ( empty( $ids ) ) {
			return;
		}

		if ( 'order_courier' === $action_name ) {
			$sendback = admin_url( 'admin.php?page=' . Defaults::ADMIN_PAGE . '&path=/order_courier&ids=' . implode( ',', $ids ) );
			wp_redirect( $sendback );
			exit();
		}
	}

	/**
	 * Back to my parcels page
	 *
	 * @param  string $error Error.
	 * @return void
	 */
	private function back_to_my_parcels( string $error ) {
		Notice::get_instance()->add( 'error', $error );
		$this->action_my_parcels();
	}
}
