<?php
/**
 * Class CheckoutController.
 *
 * @package ILabs\WoocommerceDHL24
 */

declare(strict_types=1);
namespace ILabs\WoocommerceDHL24\Controller;

use WC_Checkout;
use ILabs\WoocommerceDHL24\Helper\Param;
use ILabs\WoocommerceDHL24\Helper\Option;
use ILabs\WoocommerceDHL24\Helper\Normalizer;
use ILabs\WoocommerceDHL24\Helper\NearestService;
use ILabs\WoocommerceDHL24\Exception\ExceptionInterface;
use ILabs\WoocommerceDHL24\Helper\CreateShipmentContext;

/**
 * CheckoutController
 */
class CheckoutController extends AbstractController implements ControllerInterface {

	/**
	 * Normalizer
	 *
	 * @var Normalizer
	 */
	private $normalizer;

	/**
	 * Init class.
	 */
	public function init() {
		$this->normalizer = new Normalizer();

		add_action( 'wp_enqueue_scripts', array( $this, 'wp_enqueue_scripts' ) );

		add_action( 'woocommerce_after_checkout_form', array( $this, 'after_checkout_form' ) );
		add_action( 'woocommerce_before_order_notes', array( $this, 'before_order_notes' ) );
		add_action( 'woocommerce_review_order_before_submit', array( $this, 'review_order_before_submit' ) );
		add_action( 'woocommerce_checkout_update_order_meta', array( $this, 'update_order_meta' ), 10, 2 );
	}

	/**
	 * Enqueue scripts
	 *
	 * @internal Add action hook: {@link init()}
	 * @return void
	 */
	public function wp_enqueue_scripts() {
		if ( ! is_checkout() ) {
			return;
		}

		wp_enqueue_style( 'wp-jquery-ui-dialog' );
		wp_enqueue_script( 'jquery-ui-dialog' );
		wp_enqueue_script(
			'dhl24',
			$this->plugin_options->get_plugin_url() . '/assets/js/checkout.js',
			array(),
			bin2hex( random_bytes( 16 ) ), // $this->plugin_options->get_plugin_version()
			true
		);
	}

	/**
	 * Review order before submit
	 *
	 * @return void
	 */
	public function review_order_before_submit() {
		$checkout        = WC()->checkout;
		$nearest_service = new NearestService( $this->get_shipping_method(), (string) $checkout->get_value( 'shipping_postcode' ), (string) $checkout->get_value( 'shipping_country' ) );
		$point           = $nearest_service->get_point();

		$this->render(
			'review-order-before-submit.php',
			array(
				'shipping_method' => $this->get_shipping_method(),
				'sap'             => $point->getSap(),
				'point'           => $point,
			)
		);
	}

	/**
	 * Update order meta
	 *
	 * @param  int   $order_id ID.
	 * @param  array $data Posted data.
	 * @return void
	 */
	public function update_order_meta( int $order_id, array $data ) {
		if ( ! empty( $data['shipping_method'] ) ) {
			$shipping_method = $data['shipping_method'][0] ?? '';
			$shipping_method = $this->normalizer->shipping_method( $shipping_method );

			if ( $this->is_shipping_method_dhl24( $shipping_method ) ) {
				$data['dhl24_sap'] = ! empty( $_POST['dhl24_sap'] ) ? sanitize_text_field( wp_unslash( $_POST['dhl24_sap'] ) ) : ''; // phpcs:ignore WordPress.Security.NonceVerification
				if ( $this->is_servicepoint_shipping_method( $shipping_method ) ) {
					if ( empty( $data['dhl24_sap'] ) ) {
						wc_add_notice( __( 'Select DHL POP.', 'woocommerce-dhl24' ), 'error' );
						wp_send_json(
							array(
								'result'   => 'failure',
								'messages' => wc_print_notices( true ),
								'refresh'  => false,
								'reload'   => false,
							)
						);
					}
					update_post_meta( $order_id, 'dhl24_sap', $data['dhl24_sap'] );
				}

				if ( 'yes' === Option::get_instance()->get_value( 'auto_create' ) ) {
					$data['shipping_method'] = strlen( $shipping_method ) > 2 ? strtoupper( substr( $shipping_method, -2 ) ) : '';
					if ( 'cod' === $data['payment_method'] ) {
						if ( ! isset( $data['service'] ) ) {
							$data['service'] = array();
						}
						$order                                     = wc_get_order( $order_id );
						$data['service']['insurance']              = true;
						$data['service']['insuranceValue']         = $order->get_total();
						$data['service']['collectOnDelivery']      = true;
						$data['service']['collectOnDeliveryValue'] = $order->get_total();
					}
					$create_shipment = new CreateShipmentContext( $shipping_method, $order_id, $data );

					try {
						$create_shipment->create();
					} catch ( ExceptionInterface $e ) {
						wc_add_notice( $e->getMessage(), 'error' );
						wp_send_json(
							array(
								'result'   => 'failure',
								'messages' => wc_print_notices( true ),
								'refresh'  => false,
								'reload'   => false,
							)
						);
					}
				}
			}
		}
	}

	/**
	 * Shipping to SAP
	 *
	 * @param  WC_Checkout $checkout Checkout data.
	 * @return void
	 */
	public function before_order_notes( WC_Checkout $checkout ) {
		$this->render(
			'shipping-to-sap.php'
		);
	}

	/**
	 * Shipping to SAP
	 *
	 * @param  WC_Checkout $checkout Checkout data.
	 * @return void
	 */
	public function after_checkout_form( WC_Checkout $checkout ) {
		$this->render(
			'after-checkout-form.php'
		);
	}

	/**
	 * Is shipping method DHL24
	 *
	 * @param  string $shipping_method Shipping method.
	 * @return bool
	 */
	private function is_shipping_method_dhl24( string $shipping_method ): bool {
		$allowed_shipping_methods = Param::get_instance()->get_value( 'shipping_methods' );
		if ( in_array( $shipping_method, $allowed_shipping_methods, true ) ) {
			return true;
		}
		return false;
	}

	/**
	 * Is shipping method DHL24
	 *
	 * @param  string $shipping_method Shipping method.
	 * @return bool
	 */
	private function is_servicepoint_shipping_method( string $shipping_method ): bool {
		$allowed_shipping_methods = Param::get_instance()->get_value( 'servicepoint_shipping_methods' );
		if ( in_array( $shipping_method, $allowed_shipping_methods, true ) ) {
			return true;
		}
		return false;
	}

	/**
	 * Get shipping method
	 *
	 * @return string
	 */
	private function get_shipping_method(): string {
		$shipping_methods = WC()->session->get( 'chosen_shipping_methods' );
		$shipping_method  = '';
		if ( ! empty( $shipping_methods ) ) {
			$shipping_method = $this->normalizer->shipping_method( $shipping_methods[0] );
		}

		return $shipping_method;
	}
}
