<?php
/**
 * Class DownloadController.
 *
 * @package ILabs\WoocommerceDHL24
 */

namespace ILabs\WoocommerceDHL24\Controller;

use ILabs\WoocommerceDHL24\Helper\Notice;
use ILabs\WoocommerceDHL24\Helper\Defaults;
use ILabs\WoocommerceDHL24\Exception\ExceptionInterface;
use ILabs\WoocommerceDHL24\Repository\ServicePointRepository;
use ILabs\WoocommerceDHL24\Repository\ShipmentRepository;

/**
 * DownloadController
 */
class DownloadController implements ControllerInterface {

	/**
	 * Init class.
	 */
	public function init() {
		add_action( 'init', array( $this, 'download' ) );
	}

	/**
	 * Download file
	 *
	 * @return void
	 */
	public function download() {
		if ( ! is_admin() && ! current_user_can( 'manage_options' ) ) {
			return;
		}

        // phpcs:ignore WordPress.Security.NonceVerification
		$download_label = ! empty( $_GET[ Defaults::DOWNLOAD_LABEL ] ) ? sanitize_text_field( wp_unslash( $_GET[ Defaults::DOWNLOAD_LABEL ] ) ) : '';
		if ( '' !== $download_label ) {
			$this->shipment_label( $download_label );
			return;
		}

        // phpcs:ignore WordPress.Security.NonceVerification
		$download_scan = ! empty( $_GET[ Defaults::DOWNLOAD_SCAN ] ) ? sanitize_text_field( wp_unslash( $_GET[ Defaults::DOWNLOAD_SCAN ] ) ) : '';
		if ( '' !== $download_scan ) {
			$this->shipment_scan( $download_scan );
			return;
		}
	}

	/**
	 * Shipment label
	 *
	 * @param  string $shipment_id Shipment ID.
	 * @return void
	 */
	private function shipment_label( string $shipment_id ) {
		$label_type    = ! empty( $_GET[ Defaults::LABEL_TYPE ] ) ? sanitize_text_field( wp_unslash( $_GET[ Defaults::LABEL_TYPE ] ) ) : ''; // phpcs:ignore WordPress.Security.NonceVerification
		$label_type    = strtoupper( $label_type );
		$shipment_type = ! empty( $_GET[ Defaults::SHIPMENT_TYPE ] ) ? sanitize_text_field( wp_unslash( $_GET[ Defaults::SHIPMENT_TYPE ] ) ) : ''; // phpcs:ignore WordPress.Security.NonceVerification
		$shipment_type = strtoupper( $shipment_type );

		if ( in_array( $label_type, array( 'LP', 'BLP', 'LBLP', 'ZBLP' ), true ) ) {
			if ( in_array( $shipment_type, array( 'LM', 'LP' ), true ) ) {
				$service_point_repository = new ServicePointRepository();
				try {
					$label = $service_point_repository->fetch_label_print_out( $shipment_id, $label_type );
				} catch ( ExceptionInterface $e ) {
					Notice::get_instance()->add( 'error', $e->getMessage() );
					return;
				}

                $data         = base64_decode($label->getLabelContent(), false); // phpcs:ignore
				$content_type = $label->getLabelFormat();
				$file_name    = $label->getLabelName();
			} else {
				$shipment_repository = new ShipmentRepository();
				try {
					$label = $shipment_repository->fetch_label_print_out( $shipment_id, $label_type );
				} catch ( ExceptionInterface $e ) {
					Notice::get_instance()->add( 'error', $e->getMessage() );
					return;
				}

                $data         = base64_decode($label->getLabelData(), false); // phpcs:ignore
				$content_type = $label->getLabelMimeType();
				$file_name    = $label->getShipmentId() . '_' . $label->getLabelName();
			}

			if ( false !== $data ) {
				header( 'Expires: 0' );
				header( 'Cache-Control: no-cache, no-store, must-revalidate' );
				header( 'Cache-Control: pre-check=0, post-check=0, max-age=0', false );
				header( 'Pragma: no-cache' );
				header( 'Content-type: ' . $content_type );
				header( 'Content-Disposition:attachment; filename=' . $file_name );
				header( 'Content-Type: application/force-download' );
                echo $data; // phpcs:ignore
				exit();
			}
		}

		// TODO: Add exception.
	}

	/**
	 * Shipment scan
	 *
	 * @param  string $shipment_id Shipment ID.
	 * @return void
	 */
	private function shipment_scan( string $shipment_id ) {
        // phpcs:ignore WordPress.Security.NonceVerification
		$shipment_repository = new ShipmentRepository();
		try {
			$scan = $shipment_repository->fetch_scan( $shipment_id );
		} catch ( ExceptionInterface $e ) {
			Notice::get_instance()->add( 'error', $e->getMessage() );
			return;
		}

        $data = base64_decode($scan->getScanData(), false); // phpcs:ignore
		if ( false !== $data ) {
			$mime_type      = $scan->getScanMimeType();
			$file_extension = '.png';
			if ( 'image_jpeg' === $mime_type ) {
				$file_extension = '.jpg';
			}

			header( 'Expires: 0' );
			header( 'Cache-Control: no-cache, no-store, must-revalidate' );
			header( 'Cache-Control: pre-check=0, post-check=0, max-age=0', false );
			header( 'Pragma: no-cache' );
			header( 'Content-type: ' . $scan->getScanMimeType() );
			header( 'Content-Disposition:attachment; filename=skan_' . $shipment_id . $file_extension );
			header( 'Content-Type: application/force-download' );
            echo $data; // phpcs:ignore
			exit();
		}

		// TODO: Add exception.
	}
}
