<?php
/**
 * Class CreateShipment
 *
 * @package ILabs\WoocommerceDHL24
 */

namespace ILabs\WoocommerceDHL24\DHL24Client\ServicePoint\Method;

use ILabs\WoocommerceDHL24\DHL24Client\RequestInterface;
use ILabs\WoocommerceDHL24\DHL24Client\ServicePoint\Type\Ship;
use ILabs\WoocommerceDHL24\DHL24Client\ServicePoint\Type\AuthData;
use ILabs\WoocommerceDHL24\DHL24Client\ServicePoint\Type\ShipmentInfo;
use ILabs\WoocommerceDHL24\DHL24Client\ServicePoint\Type\Piece;

/**
 * CreateShipment
 */
class CreateShipment extends AbstractMethod implements RequestInterface {
	/**
	 * Ship.
	 *
	 * @var Ship|null
	 */
	private $ship;

	/**
	 * ShipmentInfo.
	 *
	 * @var ShipmentInfo|null
	 */
	private $shipmentInfo;

	/**
	 * Parcel data, list of Piece elements.
	 *
	 * @var \ArrayObject|Piece[]
	 */
	private $pieceList;

	/**
	 * Content of the parcel.
	 *
	 * @var string|null
	 */
	private $content;

	/**
	 * Additional comments to be printed on the consignment note.
	 *
	 * @var string|null
	 */
	private $comment;

	/**
	 * Shipment reference number.
	 *
	 * @var string|null
	 */
	private $reference;

	/**
	 * Constructor
	 *
	 * @param AuthData $authData Authorization data.
	 */
	public function __construct( AuthData $authData ) {
		parent::__construct( $authData );

		$this->pieceList = new \ArrayObject();
	}

	/**
	 * Return associative array.
	 *
	 * @return array
	 */
	public function toArray(): array {
		$pieceList = array();
		foreach ( $this->getPieceList() as $key => $piece ) {
			$pieceList[] = $piece->toArray();
		}
		return array(
			'shipment' => array_merge(
				parent::toArray(),
				array(
					'shipmentData' => array(
						'ship'         => $this->getShip() ? $this->getShip()->toArray() : null,
						'shipmentInfo' => $this->getShipmentInfo() ? $this->getShipmentInfo()->toArray() : null,
						'pieceList'    => $pieceList,
						'content'      => $this->getContent(),
						'comment'      => $this->getComment(),
						'reference'    => $this->getReference(),
					),
				)
			),
		);
	}

	/**
	 * Get ship.
	 *
	 * @return Ship|null
	 */
	public function getShip() {
		return $this->ship;
	}

	/**
	 * Set ship.
	 *
	 * @param Ship|null $ship Ship.
	 *
	 * @return self
	 */
	public function setShip( $ship ) {
		$this->ship = $ship;

		return $this;
	}

	/**
	 * Get shipmentInfo.
	 *
	 * @return ShipmentInfo|null
	 */
	public function getShipmentInfo() {
		return $this->shipmentInfo;
	}

	/**
	 * Set shipmentInfo.
	 *
	 * @param ShipmentInfo|null $shipmentInfo ShipmentInfo.
	 *
	 * @return self
	 */
	public function setShipmentInfo( $shipmentInfo ) {
		$this->shipmentInfo = $shipmentInfo;

		return $this;
	}

	/**
	 * Get parcel data, list of Piece elements.
	 *
	 * @return \ArrayObject|Piece[]
	 */
	public function getPieceList() {
		return $this->pieceList;
	}

	/**
	 * Add piece.
	 *
	 * @param Piece $piece Piece.
	 *
	 * @return self
	 */
	public function addPiece( Piece $piece ) {
		$this->pieceList->append( $piece );

		return $this;
	}

	/**
	 * Remove piece.
	 *
	 * @param Piece $piece Piece.
	 *
	 * @return self
	 */
	public function removePiece( Piece $piece ) {
		// TODO.

		return $this;
	}

	/**
	 * Get content of the parcel.
	 *
	 * @return string|null
	 */
	public function getContent() {
		return $this->content;
	}

	/**
	 * Set content of the parcel.
	 *
	 * @param string|null $content Content of the parcel.
	 *
	 * @return self
	 */
	public function setContent( $content ) {
		$this->content = $content;

		return $this;
	}

	/**
	 * Get additional comments to be printed on the consignment note.
	 *
	 * @return string|null
	 */
	public function getComment() {
		return $this->comment;
	}

	/**
	 * Set additional comments to be printed on the consignment note.
	 *
	 * @param string|null $comment Additional comments to be printed on the consignment note.
	 *
	 * @return self
	 */
	public function setComment( $comment ) {
		$this->comment = $comment;

		return $this;
	}

	/**
	 * Get shipment reference number.
	 *
	 * @return string|null
	 */
	public function getReference() {
		return $this->reference;
	}

	/**
	 * Set shipment reference number.
	 *
	 * @param string|null $reference Shipment reference number.
	 *
	 * @return self
	 */
	public function setReference( $reference ) {
		$this->reference = $reference;

		return $this;
	}
}
