<?php
/**
 * Abstract class SoapClient
 *
 * @package ILabs\WoocommerceDHL24
 */

namespace ILabs\WoocommerceDHL24\DHL24Client;

/**
 * Abstract class SoapClient
 */
abstract class SoapClient extends \SoapClient implements SoapClientInterface {

	/**
	 * Options.
	 *
	 * @var array
	 */
	protected $options;
	/**
	 * Mock requests.
	 *
	 * @var array
	 */
	private $mock_requests = array();
	/**
	 * Mock responses.
	 *
	 * @var array
	 */
	private $mock_responses = array();


	/**
	 * The name of the SOAP function to call.
	 *
	 * @var string
	 */
	protected $method;

	/**
	 * SoapClient constructor.
	 *
	 * @param string|null $wsdl URI of the WSDL file or null if working in non-WSDL mode.
	 * @param array       $options Options.
	 */
	public function __construct( $wsdl = null, array $options = array() ) {

		if ( isset( $options['mock_requests'] ) ) {
			$this->set_mock_requests( $options['mock_requests'] );
			unset( $options['mock_requests'] );
		}
		if ( isset( $options['mock_responses'] ) ) {
			$this->set_mock_responses( $options['mock_responses'] );
			unset( $options['mock_responses'] );
		}
		$context = stream_context_create( array( 'http' => array( 'header' => 'DHLApiOrgin: WooCommerce' ) ) );

		$defaults = array(
			'compression'        => ( SOAP_COMPRESSION_ACCEPT | SOAP_COMPRESSION_GZIP ),
			'cache_wsdl'         => WSDL_CACHE_BOTH,
			'connection_timeout' => 60,
			'exceptions'         => true,
			'features'           => SOAP_SINGLE_ELEMENT_ARRAYS,
			'soap_version'       => SOAP_1_2,
			'trace'              => true,
			'stream_context'     => stream_context_create( array( 'http' => array( 'header' => 'DHLApiOrgin: WooCommerce' ) ) ),
			'user_agent'         => 'inspirelabs/woocommerce-dhl24',
		);

		$options = array_merge( $defaults, $options );

		parent::__construct( $wsdl, $options );
		$this->options = $options;
	}

	/**
	 * { @inheritDoc }
	 *
	 * @return array
	 */
	public function get_options(): array {
		return $this->options;
	}

	/**
	 * __call
	 *
	 * @param  string $function_name The name of the SOAP function to call.
	 * @param  array  $arguments An array of the arguments to pass to the function.
	 *
	 * @throws \Exception If error.
	 *
	 * @return mixed
	 */
	public function __call( $function_name, $arguments ) {
		try {
			$response = parent::__call( $function_name, $arguments );
			// works only with 'exceptions' => false, we always throw.
			if ( is_soap_fault( $response ) ) {
				throw $response;
			}
		} catch ( \Exception $e ) {
			$this->handle_fault( $function_name, $arguments, $e );
		}

		return $response;
	}

	/**
	 * Calls a SOAP function.
	 *
	 * @param string                $function_name The name of the SOAP function to call.
	 * @param array                 $arguments An array of the arguments to pass to the function.
	 * @param array|null            $options An associative array of options to pass to the client.
	 * @param SoapHeader|array|null $input_headers An array of headers to be sent along with the SOAP request.
	 * @param array|null            $output_headers If supplied, this array will be filled with the headers from the SOAP response.
	 *
	 * @throws \Exception If error.
	 *
	 * @return mixed
	 */
	public function __soapCall(
		$function_name,
		$arguments,
		$options = null,
		$input_headers = null,
		&$output_headers = null
	) {
		try {
			$response = parent::__soapCall( $function_name, $arguments, $options, $input_headers, $output_headers );
			// Works only with 'exceptions' => false, we always throw.
			if ( is_soap_fault( $response ) ) {
				throw $response;
			}
		} catch ( \Exception $e ) {
			$this->handle_fault( $function_name, $arguments, $e );
		}

		return $response;
	}

	/**
	 * { @inheritDoc }
	 *
	 * @param  string $request The XML SOAP request.
	 * @param  string $location The URL to request.
	 * @param  string $action The SOAP action.
	 * @param  int    $version The SOAP version.
	 * @param  mixed  $one_way If one_way is set to 1, this method returns nothing.
	 * @return bool|string
	 */
	public function __doRequest( $request, $location, $action, $version, $one_way = false ) {
		// Performs mocks.
		foreach ( $this->mock_requests as $key => $mock_request ) {
			if ( is_string( $key ) ) {
				if ( true === $this->str_ends_with( $action, $key ) ) {
					$request = file_get_contents( $mock_request );
					break;
				}
			} else {
				if ( is_callable( $mock_request ) ) {
					$requestFilePath = $mock_request( $request, $location, $action, $version, $one_way );
					if ( ! empty( $requestFilePath ) ) {
						$request = file_get_contents( $requestFilePath );
						break;
					}
				}
			}
		}

		foreach ( $this->mock_responses as $key => $mock_response ) {
			if ( is_string( $key ) ) {
				if ( true === $this->str_ends_with( $action, $key ) ) {
					$response = file_get_contents( $mock_response );

					return $response;
				}
			} else {
				if ( is_callable( $mock_response ) ) {
					$responseFilePath = $mock_response( $request, $location, $action, $version, $one_way );
					if ( ! empty( $requestFilePath ) ) {
						$response = file_get_contents( $responseFilePath );

						return $response;
					}
				}
			}
		}

		/* workaround for working timeout */
		$socketTimeout = false;
		if ( isset( $this->options['connection_timeout'] ) ) {
			$socketTimeout = ini_set( 'default_socket_timeout', $this->options['connection_timeout'] );
		}

		$response = parent::__doRequest( $request, $location, $action, $version, $one_way );

		if ( false !== $socketTimeout ) {
			ini_set( 'default_socket_timeout', $socketTimeout );
		}

		return $response;
	}

	/**
	 * { @inheritDoc }
	 *
	 * @param array $mock_requests Mock requests.
	 */
	public function set_mock_requests( array $mock_requests ) {
		$this->mock_requests = $mock_requests;
	}

	/**
	 * { @inheritDoc }
	 *
	 * @param array $mock_responses Mock responses.
	 */
	public function set_mock_responses( array $mock_responses ) {
		$this->mock_responses = $mock_responses;
	}

	/**
	 * Handle fault.
	 *
	 * @param string     $function_name The name of the SOAP function to call.
	 * @param array      $arguments An array of the arguments to pass to the function.
	 * @param \SoapFault $e Represents a SOAP fault.
	 *
	 * @throws \Exception If error.
	 */
	protected function handle_fault( string $function_name, array $arguments, $e ) {
		if ( defined( 'WP_DEBUG' ) && WP_DEBUG ) {
			// TODO: Error log.
			$log_dir = __DIR__ . '/../../var/error_';
			file_put_contents( $log_dir . $this->method . 'Request.xml', $this->anonymize( $this->__getLastRequest() ) );
			file_put_contents( $log_dir . $this->method . 'Response.xml', $this->__getLastResponse() );
			file_put_contents( $log_dir . $this->method . '.txt', print_r( $arguments, true ) );
		}

		throw $e;
	}

	/**
	 * Call SOAP function.
	 *
	 * @param RequestInterface $request The arguments to pass to the function.
	 *
	 * @throws \Exception If error.
	 *
	 * @return ResultInterface Result.
	 */
	protected function call( RequestInterface $request ): ResultInterface {
		try {
			$result = $this->__soapCall( $this->method, array( $request->toArray() ) );
		} catch ( \Exception $e ) {
			$this->handle_fault( $this->method, array( $request ), $e );
		}

		if ( defined( 'WP_DEBUG' ) && WP_DEBUG ) {
			// TODO: Make logger.
			$log_dir = __DIR__ . '/../../var/';
			file_put_contents( $log_dir . $this->method . 'Request.xml', $this->anonymize( $this->__getLastRequest() ) );
			file_put_contents( $log_dir . $this->method . 'Response.xml', $this->__getLastResponse() );
		}

		return $this->cast( $result );
	}

	/**
	 * Class result
	 *
	 * @param \stdClass $sourceObject Source object.
	 *
	 * @return ResultInterface
	 */
	abstract protected function cast( \stdClass $sourceObject ): ResultInterface;

	/**
	 * Checks if a string ends with a given substring.
	 *
	 * @param  string $haystack Haystack.
	 * @param  string $needle Needle.
	 * @return bool
	 */
	private function str_ends_with( string $haystack, string $needle ): bool {
		$needle_len = strlen( $needle );
		return ( 0 === $needle_len || 0 === substr_compare( $haystack, $needle, - $needle_len ) );
	}

	/**
	 * Anonymize data.
	 *
	 * @param  string $data Data string.
	 * @return string
	 */
	private function anonymize( string $data ): string {
		return preg_replace(
			array(
				'/<authData>(.*)<\/authData>/mi',
				'/<accountNumber>(.*)<\/accountNumber>/mi',
			),
			array(
				'<authData xsi:nil="true"/>',
				'<accountNumber xsi:nil="true"/>',
			),
			$data
		);
	}
}
