<?php
/**
 * Class BookCourier
 *
 * @package ILabs\WoocommerceDHL24
 */

namespace ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Method;

use ILabs\WoocommerceDHL24\DHL24Client\RequestInterface;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\Strings;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\AuthData;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\ShipmentOrderInfo;

/**
 * BookCourier
 */
class BookCourier extends AbstractMethod implements RequestInterface {
	/**
	 * Pickup date.
	 *
	 * @var \DateTime
	 */
	private $pickupDate;

	/**
	 * Time from which the shipment is ready to be collected (in HH:MM format).
	 *
	 * @var string
	 */
	private $pickupTimeFrom;

	/**
	 * Time until which the shipment can be collected (in HH:MM format).
	 *
	 * @var string
	 */
	private $pickupTimeTo;

	/**
	 * Additional information for the courier.
	 *
	 * @var string
	 */
	private $additionalInfo;

	/**
	 * Array of string-type items containing shipment IDs.
	 *
	 * @var \ArrayObject|string[]
	 */
	private $shipmentIdList;

	/**
	 * Summary data on the sender and the number and weight of shipments.
	 *
	 * @var ShipmentOrderInfo
	 */
	private $shipmentOrderInfo;

	/**
	 * Should the courier come with a label.
	 *
	 * @var bool|null
	 */
	private $courierWithLabel;

	/**
	 * Constructor
	 *
	 * @param AuthData $authData Authorization data.
	 */
	public function __construct( AuthData $authData ) {
		parent::__construct( $authData );

		$this->pickupDate     = new \DateTime( 'tomorrow' );
		$this->shipmentIdList = new \ArrayObject();
	}

	/**
	 * Return associative array.
	 *
	 * @return array
	 */
	public function toArray(): array {
		$shipmentIdList = array();
		foreach ( $this->getShipmentIdList() as $key => $value ) {
			$shipmentIdList[] = $value;
		}
		return array_merge(
			parent::toArray(),
			array(
				'pickupDate'        => $this->getPickupDate()->format( 'Y-m-d' ), // Date in YYYY-MM-DD format.
				'pickupTimeFrom'    => $this->getPickupTimeFrom(),
				'pickupTimeTo'      => $this->getPickupTimeTo(),
				'additionalInfo'    => $this->getAdditionalInfo(),
				'shipmentIdList'    => $shipmentIdList,
				'shipmentOrderInfo' => $this->getShipmentOrderInfo() ? $this->getShipmentOrderInfo()->toArray() : null,
				'courierWithLabel'  => $this->getCourierWithLabel(),
			)
		);
	}

	/**
	 * Get pickup date.
	 *
	 * @return \DateTime
	 */
	public function getPickupDate(): \DateTime {
		return $this->pickupDate;
	}

	/**
	 * Set pickup date.
	 *
	 * @param \DateTime $pickupDate Pickup date.
	 *
	 * @return self
	 */
	public function setPickupDate( \DateTime $pickupDate ) {
		$this->pickupDate = $pickupDate;

		return $this;
	}

	/**
	 * Get time from which the shipment is ready to be collected (in HH:MM format).
	 *
	 * @return string
	 */
	public function getPickupTimeFrom() {
		return $this->pickupTimeFrom;
	}

	/**
	 * Set time from which the shipment is ready to be collected (in HH:MM format).
	 *
	 * @param string $pickupTimeFrom Time from which the shipment is ready to be collected (in HH:MM format).
	 *
	 * @return self
	 */
	public function setPickupTimeFrom( string $pickupTimeFrom ) {
		$this->pickupTimeFrom = $pickupTimeFrom;

		return $this;
	}

	/**
	 * Get time until which the shipment can be collected (in HH:MM format).
	 *
	 * @return string
	 */
	public function getPickupTimeTo() {
		return $this->pickupTimeTo;
	}

	/**
	 * Set time until which the shipment can be collected (in HH:MM format).
	 *
	 * @param string $pickupTimeTo Time until which the shipment can be collected (in HH:MM format).
	 *
	 * @return self
	 */
	public function setPickupTimeTo( string $pickupTimeTo ) {
		$this->pickupTimeTo = $pickupTimeTo;

		return $this;
	}

	/**
	 * Get additional information for the courier.
	 *
	 * @return string
	 */
	public function getAdditionalInfo() {
		return $this->additionalInfo;
	}

	/**
	 * Set additional information for the courier.
	 *
	 * @param string $additionalInfo Additional information for the courier.
	 *
	 * @return self
	 */
	public function setAdditionalInfo( string $additionalInfo ) {
		$this->additionalInfo = $additionalInfo;

		return $this;
	}

	/**
	 * Get array of string-type items containing shipment IDs.
	 *
	 * @return Strings
	 */
	public function getShipmentIdList() {
		return $this->shipmentIdList;
	}

	/**
	 * Add shipment ID.
	 *
	 * @param string $shipmentId Shipment ID.
	 *
	 * @return self
	 */
	public function addShipmentId( string $shipmentId ) {
		$this->shipmentIdList->append( $shipmentId );

		return $this;
	}

	/**
	 * Remove shipment ID.
	 *
	 * @param string $shipmentId Shipment ID.
	 *
	 * @return self
	 */
	public function removeShipmentId( string $shipmentId ) {
		// TODO.

		return $this;
	}

	/**
	 * Get summary data on the sender and the number and weight of shipments.
	 *
	 * @return ShipmentOrderInfo
	 */
	public function getShipmentOrderInfo() {
		return $this->shipmentOrderInfo;
	}

	/**
	 * Set summary data on the sender and the number and weight of shipments.
	 *
	 * @param ShipmentOrderInfo $shipmentOrderInfo Summary data on the sender and the number and weight of shipments.
	 *
	 * @return self
	 */
	public function setShipmentOrderInfo( ShipmentOrderInfo $shipmentOrderInfo ) {
		$this->shipmentOrderInfo = $shipmentOrderInfo;

		return $this;
	}

	/**
	 * Get should the courier come with a label.
	 *
	 * @return bool|null
	 */
	public function getCourierWithLabel() {
		return $this->courierWithLabel;
	}

	/**
	 * Set should the courier come with a label.
	 *
	 * @param bool|null $courierWithLabel Should the courier come with a label.
	 *
	 * @return self
	 */
	public function setCourierWithLabel( $courierWithLabel ) {
		$this->courierWithLabel = $courierWithLabel;

		return $this;
	}
}
