<?php
/**
 * Class CreateShipmentReturn
 *
 * @package ILabs\WoocommerceDHL24
 */

namespace ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Method;

use ILabs\WoocommerceDHL24\DHL24Client\RequestInterface;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\AuthData;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\ReturnShip;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\ReturnPackage;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\ReturnShipmentInfo;

/**
 * CreateShipmentReturn
 */
class CreateShipmentReturn extends AbstractMethod implements RequestInterface {
	/**
	 * ReturnShipmentInfo.
	 *
	 * @var ReturnShipmentInfo|null
	 */
	private $shipmentInfo;

	/**
	 * Content of the parcel.
	 *
	 * @var string|null
	 */
	private $content;

	/**
	 * Additional comments to be printed on the consignment note.
	 *
	 * @var string|null
	 */
	private $comment;

	/**
	 * Shipment reference number.
	 *
	 * @var string|null
	 */
	private $reference;

	/**
	 * Original shipment number.
	 *
	 * @var string|null
	 */
	private $primaryWaybillNumber;

	/**
	 * ReturnShip.
	 *
	 * @var ReturnShip|null
	 */
	private $ship;

	/**
	 * Parcel data, list of ReturnPackage elements.
	 *
	 * @var \ArrayObject|ReturnPackage[]
	 */
	private $pieceList;

	/**
	 * Constructor
	 *
	 * @param AuthData $authData Authorization data.
	 */
	public function __construct( AuthData $authData ) {
		parent::__construct( $authData );

		$this->pieceList = new \ArrayObject();
	}

	/**
	 * Return associative array.
	 *
	 * @return array
	 */
	public function toArray(): array {
		$pieceList = array();
		foreach ( $this->getPieceList() as $key => $piece ) {
			$pieceList[] = $piece->toArray();
		}
		return array_merge(
			parent::toArray(),
			array(
				'shipment' => array(
					'shipmentInfo'         => $this->getShipmentInfo() ? $this->getShipmentInfo()->toArray() : null,
					'content'              => $this->getContent(),
					'comment'              => $this->getComment(),
					'reference'            => $this->getReference(),
					'primaryWaybillNumber' => $this->getPrimaryWaybillNumber(),
					'ship'                 => $this->getShip() ? $this->getShip()->toArray() : null,
					'pieceList'            => $pieceList,
				),
			)
		);
	}

	/**
	 * Get shipmentInfo.
	 *
	 * @return ReturnShipmentInfo|null
	 */
	public function getShipmentInfo() {
		return $this->shipmentInfo;
	}

	/**
	 * Set shipmentInfo.
	 *
	 * @param ReturnShipmentInfo|null $shipmentInfo ReturnShipmentInfo.
	 *
	 * @return self
	 */
	public function setShipmentInfo( $shipmentInfo ) {
		$this->shipmentInfo = $shipmentInfo;

		return $this;
	}

	/**
	 * Get content of the parcel.
	 *
	 * @return string|null
	 */
	public function getContent() {
		return $this->content;
	}

	/**
	 * Set content of the parcel.
	 *
	 * @param string|null $content Content of the parcel.
	 *
	 * @return self
	 */
	public function setContent( $content ) {
		$this->content = $content;

		return $this;
	}

	/**
	 * Get additional comments to be printed on the consignment note.
	 *
	 * @return string|null
	 */
	public function getComment() {
		return $this->comment;
	}

	/**
	 * Set additional comments to be printed on the consignment note.
	 *
	 * @param string|null $comment Additional comments to be printed on the consignment note.
	 *
	 * @return self
	 */
	public function setComment( $comment ) {
		$this->comment = $comment;

		return $this;
	}

	/**
	 * Get shipment reference number.
	 *
	 * @return string|null
	 */
	public function getReference() {
		return $this->reference;
	}

	/**
	 * Set shipment reference number.
	 *
	 * @param string|null $reference Shipment reference number.
	 *
	 * @return self
	 */
	public function setReference( $reference ) {
		$this->reference = $reference;

		return $this;
	}

	/**
	 * Get original shipment number.
	 *
	 * @return  string|null
	 */
	public function getPrimaryWaybillNumber() {
		return $this->primaryWaybillNumber;
	}

	/**
	 * Set original shipment number.
	 *
	 * @param  string|null $primaryWaybillNumber  Original shipment number.
	 *
	 * @return  self
	 */
	public function setPrimaryWaybillNumber( $primaryWaybillNumber ) {
		$this->primaryWaybillNumber = $primaryWaybillNumber;

		return $this;
	}

	/**
	 * Get ship.
	 *
	 * @return ReturnShip|null
	 */
	public function getShip() {
		return $this->ship;
	}

	/**
	 * Set ship.
	 *
	 * @param ReturnShip|null $ship ReturnShip.
	 *
	 * @return self
	 */
	public function setShip( $ship ) {
		$this->ship = $ship;

		return $this;
	}

	/**
	 * Get parcel data, list of ReturnPackage elements.
	 *
	 * @return \ArrayObject|ReturnPackage[]
	 */
	public function getPieceList() {
		return $this->pieceList;
	}

	/**
	 * Add piece.
	 *
	 * @param ReturnPackage $returnPackage ReturnPackage.
	 *
	 * @return self
	 */
	public function addPiece( ReturnPackage $returnPackage ) {
		$this->pieceList->append( $returnPackage );

		return $this;
	}

	/**
	 * Remove piece.
	 *
	 * @param ReturnPackage $returnPackage ReturnPackage.
	 *
	 * @return self
	 */
	public function removePiece( ReturnPackage $returnPackage ) {
		// TODO.

		return $this;
	}
}
