<?php
/**
 * Class ReturnShipmentInfo
 *
 * @package ILabs\WoocommerceDHL24
 */

namespace ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type;

/**
 * ReturnShipmentInfo
 */
class ReturnShipmentInfo {
	/**
	 * Shipment waybill number.
	 *
	 * @var string|null
	 */
	private $wayBill;

	/**
	 * Service type.
	 * Dictionary value; possible values:
	 * ZK - ZK
	 *
	 * @var string|null
	 */
	private $serviceType;

	/**
	 * The option of shipping a return shipment with a courier collection.
	 *
	 * @var bool
	 */
	private $bookCourier;

	/**
	 * Billing information.
	 *
	 * @var Billing|null
	 */
	private $billing;

	/**
	 * Ordering special services.
	 *
	 * @var \ArrayObject|ReturnService[]
	 */
	private $specialServices;

	/**
	 * Information on the parcel shipment time.
	 *
	 * @var ReturnShipmentTime|null
	 */
	private $shipmentTime;

	/**
	 * Return label selection.
	 *
	 * @var string|null
	 */
	private $labelType;

	/**
	 * Return associative array.
	 *
	 * @return array
	 */
	public function toArray(): array {
		$specialServices = array();
		foreach ( $this->getSpecialServices() as $item ) {
			$specialServices[] = $item->toArray();
		}
		return array(
			'wayBill'         => $this->getWayBill(),
			'serviceType'     => $this->getServiceType(),
			'bookCourier'     => $this->getBookCourier(),
			'billing'         => $this->getBilling() ? $this->getBilling()->toArray() : null,
			'specialServices' => $specialServices,
			'shipmentTime'    => $this->getShipmentTime() ? $this->getShipmentTime()->toArray() : null,
			'labelType'       => $this->getLabelType(),
		);
	}

	/**
	 * Construct
	 *
	 * @return void
	 */
	public function __construct() {
		$this->specialServices = new \ArrayObject();
	}

	/**
	 * Get shipment waybill number.
	 *
	 * @return string|null
	 */
	public function getWayBill() {
		return $this->wayBill;
	}

	/**
	 * Set shipment waybill number.
	 *
	 * @param string|null $wayBill Shipment waybill number.
	 *
	 * @return self
	 */
	public function setWayBill( $wayBill ) {
		$this->wayBill = $wayBill;

		return $this;
	}

	/**
	 * Get service type.
	 *
	 * @return string|null
	 */
	public function getServiceType() {
		return $this->serviceType;
	}

	/**
	 * Set service type.
	 *
	 * @param string|null $serviceType Type of shipping service.
	 *
	 * @return self
	 */
	public function setServiceType( $serviceType ) {
		$this->serviceType = $serviceType;

		return $this;
	}

	/**
	 * Get book courier.
	 *
	 * @return  bool
	 */
	public function getBookCourier() {
		return $this->bookCourier;
	}

	/**
	 * Set book courier.
	 *
	 * @param  bool $bookCourier  Book courier.
	 *
	 * @return  self
	 */
	public function setBookCourier( bool $bookCourier ) {
		$this->bookCourier = $bookCourier;

		return $this;
	}

	/**
	 * Get billing information.
	 *
	 * @return Billing|null
	 */
	public function getBilling() {
		return $this->billing;
	}

	/**
	 * Set billing information.
	 *
	 * @param Billing|null $billing Billing information.
	 *
	 * @return self
	 */
	public function setBilling( $billing ) {
		$this->billing = $billing;

		return $this;
	}

	/**
	 * Get ordering special services.
	 *
	 * @return \ArrayObject|ReturnService[]
	 */
	public function getSpecialServices() {
		return $this->specialServices;
	}

	/**
	 * Add ordering special service.
	 *
	 * @param ReturnService $specialService Ordering special service.
	 *
	 * @return self
	 */
	public function addSpecialServices( ReturnService $specialService ) {
		$this->specialServices->append( $specialService );

		return $this;
	}

	/**
	 * Remove ordering special service.
	 *
	 * @param ReturnService $specialService Ordering special service.
	 *
	 * @return self
	 */
	public function removeSpecialServices( ReturnService $specialService ) {
		// TODO.

		return $this;
	}

	/**
	 * Get information on the parcel shipment time.
	 *
	 * @return ReturnShipmentTime|null
	 */
	public function getShipmentTime() {
		return $this->shipmentTime;
	}

	/**
	 * Set information on the parcel shipment time.
	 *
	 * @param ReturnShipmentTime|null $shipmentTime Information on the parcel shipment time.
	 *
	 * @return self
	 */
	public function setShipmentTime( $shipmentTime ) {
		$this->shipmentTime = $shipmentTime;

		return $this;
	}

	/**
	 * Get return label selection.
	 *
	 * @return string|null
	 */
	public function getLabelType() {
		return $this->labelType;
	}

	/**
	 * Set return label selection.
	 *
	 * @param string|null $labelType Return label selection.
	 *
	 * @return self
	 */
	public function setLabelType( $labelType ) {
		$this->labelType = $labelType;

		return $this;
	}
}
