<?php
/**
 * Class ShipmentBasicDatas
 *
 * @package ILabs\WoocommerceDHL24
 */

namespace ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type;

/**
 * ShipmentFullData
 */
class ShipmentFullData {
	/**
	 * Sender data communicated in the address structure.
	 *
	 * @var Address|null
	 */
	private $shipper;

	/**
	 * Recipient data communicated in the address structure.
	 *
	 * @var ReceiverAddressData|null
	 */
	private $receiver;

	/**
	 * Neighbour address.
	 *
	 * @var NeighbourAddress|null
	 */
	private $neighbour;

	/**
	 * Parcel data, list of PieceDefinition elements.
	 *
	 * @var \ArrayObject|PieceDefinition[]
	 */
	private $pieceList;

	/**
	 * Payer and payment data.
	 *
	 * @var Payment|null
	 */
	private $payment;

	/**
	 * Data on the selected shipping service and special services.
	 *
	 * @var ServiceDefinition|null
	 */
	private $service;

	/**
	 * Shipping date (visible on the consignment note) - in YYYY-MM-DD format.
	 *
	 * @var \DateTime
	 */
	private $shipmentDate;

	/**
	 * Should the restriction check be omitted?
	 *
	 * @var bool|null
	 */
	private $skipRestrictionCheck;

	/**
	 * Additional comments (visible on the consignment note).
	 *
	 * @var string|null
	 */
	private $comment;

	/**
	 * Content of the parcel.
	 *
	 * @var string|null
	 */
	private $content;

	/**
	 * Shipment reference number.
	 *
	 * @var string|null
	 */
	private $reference;

	/**
	 * Response only Shipment ID (if the shipment has already been created, otherwise field left blank).
	 *
	 * @var string
	 */
	private $shipmentId;

	/**
	 * Response only Shipment creation date in YYYY-MM-DD format (if the shipment has already been created, otherwise field left blank).
	 *
	 * @var string|null
	 */
	private $created;

	/**
	 * Response only Information whether a courier is booked for this shipment.
	 *
	 * @var string|null
	 */
	private $orderStatus;

	/**
	 * Data on customs clearance.
	 *
	 * @var CustomsData|null
	 */
	private $customs;

	/**
	 * Shipment waybill number.
	 *
	 * @var string|null
	 */
	private $wayBill;

	/**
	 * Construct
	 *
	 * @return void
	 */
	public function __construct() {
		$this->pieceList = new \ArrayObject();
		$this->shipmentDate = new \DateTime( 'tomorrow' );
		$this->shipmentId = '';
	}

	/**
	 * Return associative array.
	 *
	 * @return array
	 */
	public function toArray(): array {
		$pieceList = array();
		foreach ( $this->getPieceList() as $key => $piece ) {
			$pieceList[] = $piece->toArray();
		}
		return array(
			'shipper'              => $this->getShipper() ? $this->getShipper()->toArray() : null,
			'receiver'             => $this->getReceiver() ? $this->getReceiver()->toArray() : null,
			'neighbour'            => $this->getNeighbour() ? $this->getNeighbour()->toArray() : null,
			'pieceList'            => $pieceList,
			'payment'              => $this->getPayment() ? $this->getPayment()->toArray() : null,
			'service'              => $this->getService() ? $this->getService()->toArray() : null,
			'shipmentDate'         => $this->getShipmentDate()->format( 'Y-m-d' ), // Date in YYYY-MM-DD format.
			'skipRestrictionCheck' => $this->getSkipRestrictionCheck(),
			'comment'              => $this->getComment(),
			'content'              => $this->getContent(),
			'reference'            => $this->getReference(),
			'shipmentId'           => $this->getShipmentId(),
			'created'              => $this->getCreated(),
			'orderStatus'          => $this->getOrderStatus(),
			'customs'              => $this->getCustoms() ? $this->getCustoms()->toArray() : null,
			'wayBill'              => $this->getWayBill(),
		);
	}

	/**
	 * Get sender data communicated in the address structure.
	 *
	 * @return Address|null
	 */
	public function getShipper() {
		return $this->shipper;
	}

	/**
	 * Set sender data communicated in the address structure.
	 *
	 * @param Address|null $shipper Sender data communicated in the address structure.
	 *
	 * @return self
	 */
	public function setShipper( $shipper ) {
		$this->shipper = $shipper;

		return $this;
	}

	/**
	 * Get recipient data communicated in the address structure.
	 *
	 * @return ReceiverAddressData|null
	 */
	public function getReceiver() {
		return $this->receiver;
	}

	/**
	 * Set recipient data communicated in the address structure.
	 *
	 * @param ReceiverAddressData|null $receiver Recipient data communicated in the address structure.
	 *
	 * @return self
	 */
	public function setReceiver( $receiver ) {
		$this->receiver = $receiver;

		return $this;
	}

	/**
	 * Get neighbour address.
	 *
	 * @return NeighbourAddress|null
	 */
	public function getNeighbour() {
		return $this->neighbour;
	}

	/**
	 * Set neighbour address.
	 *
	 * @param NeighbourAddress|null $neighbour Neighbour address.
	 *
	 * @return self
	 */
	public function setNeighbour( $neighbour ) {
		$this->neighbour = $neighbour;

		return $this;
	}

	/**
	 * Get parcel data, list of PieceDefinition elements.
	 *
	 * @return \ArrayObject|PieceDefinition[]
	 */
	public function getPieceList() {
		return $this->pieceList;
	}

	/**
	 * Add piece.
	 *
	 * @param PieceDefinition $pieceDefinition PieceDefinition.
	 *
	 * @return self
	 */
	public function addPiece( PieceDefinition $pieceDefinition ) {
		$this->pieceList->append( $pieceDefinition );

		return $this;
	}

	/**
	 * Remove piece.
	 *
	 * @param PieceDefinition $pieceDefinition PieceDefinition.
	 *
	 * @return self
	 */
	public function removePiece( PieceDefinition $pieceDefinition ) {
		// TODO.

		return $this;
	}

	/**
	 * Get payer and payment data.
	 *
	 * @return Payment|null
	 */
	public function getPayment() {
		return $this->payment;
	}

	/**
	 * Set payer and payment data.
	 *
	 * @param Payment|null $payment Payer and payment data.
	 *
	 * @return self
	 */
	public function setPayment( $payment ) {
		$this->payment = $payment;

		return $this;
	}

	/**
	 * Get data on the selected shipping service and special services.
	 *
	 * @return ServiceDefinition|null
	 */
	public function getService() {
		return $this->service;
	}

	/**
	 * Set data on the selected shipping service and special services.
	 *
	 * @param ServiceDefinition|null $service Data on the selected shipping service and special services.
	 *
	 * @return self
	 */
	public function setService( $service ) {
		$this->service = $service;

		return $this;
	}

	/**
	 * Get shipping date.
	 *
	 * @return \DateTime
	 */
	public function getShipmentDate(): \DateTime {
		return $this->shipmentDate;
	}

	/**
	 * Set shipping date.
	 *
	 * @param \DateTime $shipmentDate Shipping date.
	 *
	 * @return self
	 */
	public function setShipmentDate( \DateTime $shipmentDate ) {
		$this->shipmentDate = $shipmentDate;

		return $this;
	}

	/**
	 * Get should the restriction check be omitted?
	 *
	 * @return bool|null
	 */
	public function getSkipRestrictionCheck() {
		return $this->skipRestrictionCheck;
	}

	/**
	 * Set should the restriction check be omitted?
	 *
	 * @param bool|null $skipRestrictionCheck Skip restriction check.
	 *
	 * @return self
	 */
	public function setSkipRestrictionCheck( $skipRestrictionCheck ) {
		$this->skipRestrictionCheck = $skipRestrictionCheck;

		return $this;
	}

	/**
	 * Get additional comments (visible on the consignment note).
	 *
	 * @return string|null
	 */
	public function getComment() {
		return $this->comment;
	}

	/**
	 * Set additional comments (visible on the consignment note).
	 *
	 * @param string|null $comment Additional comments (visible on the consignment note).
	 *
	 * @return self
	 */
	public function setComment( $comment ) {
		$this->comment = $comment;

		return $this;
	}

	/**
	 * Get content of the parcel.
	 *
	 * @return string|null
	 */
	public function getContent() {
		return $this->content;
	}

	/**
	 * Set content of the parcel.
	 *
	 * @param string|null $content Content of the parcel.
	 *
	 * @return self
	 */
	public function setContent( $content ) {
		$this->content = $content;

		return $this;
	}

	/**
	 * Get shipment reference number.
	 *
	 * @return string|null
	 */
	public function getReference() {
		return $this->reference;
	}

	/**
	 * Set shipment reference number.
	 *
	 * @param string|null $reference Shipment reference number.
	 *
	 * @return self
	 */
	public function setReference( $reference ) {
		$this->reference = $reference;

		return $this;
	}

	/**
	 * Get response only Shipment ID (if the shipment has already been created, otherwise field left blank).
	 *
	 * @return string
	 */
	public function getShipmentId(): string {
		return $this->shipmentId;
	}

	/**
	 * Set response only Shipment ID (if the shipment has already been created, otherwise field left blank).
	 *
	 * @param string $shipmentId Response only Shipment ID (if the shipment has already been created, otherwise field left blank).
	 *
	 * @return self
	 */
	public function setShipmentId( string $shipmentId ) {
		$this->shipmentId = $shipmentId;

		return $this;
	}

	/**
	 * Get response only Shipment creation date in YYYY-MM-DD format (if the shipment has already been created, otherwise field left blank).
	 *
	 * @return string|null
	 */
	public function getCreated() {
		return $this->created;
	}

	/**
	 * Set response only Shipment creation date in YYYY-MM-DD format (if the shipment has already been created, otherwise field left blank).
	 *
	 * @param string|null $created Response only Shipment creation date in YYYY-MM-DD format (if the shipment has already been created, otherwise field left blank).
	 *
	 * @return self
	 */
	public function setCreated( $created ) {
		$this->created = $created;

		return $this;
	}

	/**
	 * Get response only Information whether a courier is booked for this shipment.
	 *
	 * @return string|null
	 */
	public function getOrderStatus() {
		return $this->orderStatus;
	}

	/**
	 * Set response only Information whether a courier is booked for this shipment.
	 *
	 * @param string|null $orderStatus Response only Information whether a courier is booked for this shipment.
	 *
	 * @return self
	 */
	public function setOrderStatus( $orderStatus ) {
		$this->orderStatus = $orderStatus;

		return $this;
	}

	/**
	 * Get data on customs clearance.
	 *
	 * @return CustomsData|null
	 */
	public function getCustoms() {
		return $this->customs;
	}

	/**
	 * Set data on customs clearance.
	 *
	 * @param CustomsData|null $customs Data on customs clearance.
	 *
	 * @return self
	 */
	public function setCustoms( $customs ) {
		$this->customs = $customs;

		return $this;
	}

	/**
	 * Get shipment waybill number.
	 *
	 * @return string|null
	 */
	public function getWayBill() {
		return $this->wayBill;
	}

	/**
	 * Set shipment waybill number.
	 *
	 * @param string|null $wayBill Shipment waybill number.
	 *
	 * @return self
	 */
	public function setWayBill( $wayBill ) {
		$this->wayBill = $wayBill;

		return $this;
	}
}
