<?php
/**
 * Class CreateShipment.
 *
 * @package ILabs\WoocommerceDHL24
 */

namespace ILabs\WoocommerceDHL24\Helper;

use ILabs\WoocommerceDHL24\Helper\Option;
use ILabs\WoocommerceDHL24\Exception\HelperException;

/**
 * CreateShipment
 */
class CreateShipmentContext {


	/**
	 * Shipping method
	 *
	 * @var string
	 */
	private $shipping_method;

	/**
	 * Strategy class
	 *
	 * @var CreateShipmentInterface
	 */
	private $strategy;

	/**
	 * Order ID
	 *
	 * @var int
	 */
	private $order_id;

	/**
	 * Posted data
	 *
	 * @var array
	 */
	private $data;

	/**
	 * Normalizer
	 *
	 * @var Normalizer
	 */
	private $normalizer;

	/**
	 * Construct
	 *
	 * @param  string $shipping_method Shipping method.
	 * @param  int    $order_id Order ID.
	 * @param  array  $data Posted data.
	 * @return void
	 */
	public function __construct( string $shipping_method, int $order_id, array $data ) {
		$this->shipping_method = $shipping_method;
		$this->order_id        = $order_id;
		$this->data            = $data;

		$this->normalizer = new Normalizer();
	}

	/**
	 * Init class
	 *
	 * @return void
	 */
	private function init() {
		$webapi_shipping_methods       = Param::get_instance()->get_value( 'webapi_shipping_methods' );
		$servicepoint_shipping_methods = Param::get_instance()->get_value( 'servicepoint_shipping_methods' );
		$return_shipping_methods       = Param::get_instance()->get_value( 'return_shipping_methods' );

		if ( ! isset( $this->data['piece'] ) || ! is_array( $this->data['piece'] ) ) {
			$data                = Option::get_instance()->get_data();
			$this->data['piece'] = array(
				array(
					'type'        => $data['piece_type'],
					'quantity'    => $data['piece_quantity'],
					'weight'      => $this->normalizer->float( $data['piece_weight'] ),
					'length'      => $data['piece_length'],
					'width'       => $data['piece_width'],
					'height'      => $data['piece_height'],
					'nonStandard' => null,
					'euroReturn'  => null,
				),
			);
		}

		if ( in_array( $this->shipping_method, $webapi_shipping_methods, true ) ) {
			// free shipping
			if( $this->shipping_method === 'free_shipping' || $this->shipping_method === 'flat_rate' ) {
				$this->data['shipping_method'] = 'AH';
			}
			$this->strategy = new CreateShipmentWebAPI( $this->order_id, $this->data );
		} elseif ( in_array( $this->shipping_method, $servicepoint_shipping_methods, true ) ) {
			$this->strategy = new CreateShipmentServicePoint( $this->order_id, $this->data );
		} elseif ( in_array( $this->shipping_method, $return_shipping_methods, true ) ) {
			$this->strategy = new CreateShipmentReturnWebAPI( $this->order_id, $this->data );
		}
	}

	/**
	 * Create shipment
	 *
	 * @return array
	 * @throws \HelperException If error.
	 */
	public function create(): array {
		$this->init();

		if ( null === $this->strategy ) {
			throw new HelperException( null, __( 'Wrong shipping method.', 'woocommerce-dhl24' ) );
		}

		$shipment_data = $this->strategy->create();

		if ( ! empty( $this->order_id ) ) {
			if ( isset( $shipment_data['shipment_id'] ) ) {
				if ( 'yes' === Option::get_instance()->get_value( 'return_auto_create' ) ) {
					$this->auto_return_shipment( $shipment_data['shipment_id'] );
				}
			}
		}

		return array(
			'fileName' => 'label.php',
			'mimeType' => $shipment_data['mimeType'],
			'fileData' => $shipment_data['fileData'],
			'id'       => $this->order_id,
		);
	}

	/**
	 * Auto return shipment
	 *
	 * @param  string $shipment_id Shipment ID.
	 * @return void
	 * @throws \HelperException If error.
	 */
	private function auto_return_shipment( string $shipment_id ) {
		$data = $this->data;
		/**
		 * Wywołanie requestu powinno wystepować tylko dla przesyłek o typie PACKAGE z wagą do 31.5kg
		 * i ENVELOPE jeśli pallet to pomimo zaznaczenia opcji proszę nie wywoływać requestu.
		 */
		foreach ( $data['piece'] as $piece ) {
			if ( 'PALLET' === $piece['type'] ) {
				return;
			}
			if ( 'PACKAGE' === $piece['type'] && 31.5 < $piece['weight'] ) {
				return;
			}
		}
		/**
		 * Jeśli produkt CM, CP, PI to nie wywoływać requestu tworzenia przesyłki zwrotnej.
		 */
		if ( in_array( $data['shipping_method'], array( 'CM', 'CP', 'PI' ), true ) ) {
			return;
		}
		if ( in_array( $data['shipping_method'], array( 'EK', 'PI', 'CP', 'CM' ), true ) ) {
			$data['shipping_method'] = 'ZC';
		} else {
			$data['shipping_method'] = 'ZK';
		}
		if ( 'yes' === Option::get_instance()->get_value( 'return_order_courier' ) ) {
			$data['book_courier'] = true;
		}

		$data['primary_waybill_number'] = $shipment_id;

		$return_shipment = new CreateShipmentReturnWebAPI( $this->order_id, $data );
		$return_shipment->create();
	}
}
