<?php
/**
 * Class CreateShipmentWebAPI.
 *
 * @package ILabs\WoocommerceDHL24
 */

namespace ILabs\WoocommerceDHL24\Helper;

use ILabs\WoocommerceDHL24\Exception\HelperException;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\Ship;
use ILabs\WoocommerceDHL24\Exception\ExceptionInterface;
use ILabs\WoocommerceDHL24\Repository\ShipmentRepository;
use ILabs\WoocommerceDHL24\Repository\ShipmentsRepository;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\Address;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\Billing;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\Payment;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\Service;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\Addressat;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\ShipmentInfo;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\ShipmentTime;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\ShipperAddress;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\PieceDefinition;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\PreavisoContact;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\ReceiverAddress;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\ShipmentFullData;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\ReceiverAddressat;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\ServiceDefinition;
use ILabs\WoocommerceDHL24\DHL24Client\WebAPI\Type\ReceiverAddressData;

/**
 * CreateShipmentWebAPI
 */
class CreateShipmentWebAPI implements CreateShipmentInterface {


	/**
	 * Order ID
	 *
	 * @var int
	 */
	private $order_id;

	/**
	 * Posted data
	 *
	 * @var array
	 */
	private $data;

	/**
	 * Is drPickup
	 *
	 * @var bool
	 */
	private $is_dr_pickup;

	/**
	 * Formatter
	 *
	 * @var Formatter
	 */
	protected $formatter;

	/**
	 * Normalizer
	 *
	 * @var Normalizer
	 */
	protected $normalizer;

	/**
	 * Services
	 *
	 * @var Services
	 */
	protected $services;

	/**
	 * Construct
	 *
	 * @param  int   $order_id Order ID.
	 * @param  array $data Posted data.
	 * @return void
	 */
	public function __construct( int $order_id, array $data ) {
		$this->order_id     = $order_id;
		$this->data         = $data;
		$this->is_dr_pickup = false;

		$this->formatter  = new Formatter();
		$this->normalizer = new Normalizer();
		$this->services   = new Services();
	}

	/**
	 * Create shipment
	 *
	 * @return array
	 * @throws \HelperException If error.
	 */
	public function create(): array {
		if ( false === $this->services->is_service_available( $this->data['shipping_method'], $this->data['shipping_country'], $this->data['shipping_postcode'] ) ) {
			throw new HelperException( null, __( 'Wrong shipping method.', 'woocommerce-dhl24' ) );
		}

		$name = $this->data['shipping_company'];
		if ( empty( $name ) ) {
			$name = $this->data['shipping_first_name'] . ' ' . $this->data['shipping_last_name'];
		}
		$street = $this->formatter->street( $this->data['shipping_address_1'] );

		$ship = new Ship();

		$shipper = new Addressat();
		$shipper->setPreaviso( $this->get_preaviso() );
		$shipper->setContact( $this->get_contact() );
		$shipper->setAddress( $this->get_shipper_address() );
		$ship->setShipper( $shipper );

		$receiver = new ReceiverAddressat();

		$receiver_address = new ReceiverAddress();
		$receiver_address->setAddressType( 'C' );
		$receiver_address->setCountry( $this->data['shipping_country'] );
		$receiver_address->setName( $name );
		$receiver_address->setPostalCode( $this->formatter->post_code( $this->data['shipping_postcode'], $this->data['shipping_country'] ) );
		$receiver_address->setCity( $this->data['shipping_city'] );
		$receiver_address->setStreet( $street['street'] );
		$receiver_address->setHouseNumber( $street['house_number'] );
		$receiver_address->setApartmentNumber( $this->data['shipping_address_2'] );
		$receiver->setAddress( $receiver_address );

		$preaviso = new PreavisoContact();
		$preaviso->setPersonName( $this->data['shipping_first_name'] . ' ' . $this->data['shipping_last_name'] );
		$preaviso->setPhoneNumber( $this->data['billing_phone'] );
		$preaviso->setEmailAddress( $this->data['billing_email'] );
		$receiver->setPreaviso( $preaviso );
		$receiver->setContact( $preaviso );

		$ship->setReceiver( $receiver );

		$pieces = array();
		if ( isset( $this->data['piece'] ) && is_array( $this->data['piece'] ) ) {
			foreach ( $this->data['piece'] as $item ) {
				$piece = new PieceDefinition();
				$piece->setType( $item['type'] );
				$piece->setQuantity( $item['quantity'] );
				$piece->setWeight( $item['weight'] );
				$piece->setLength( $item['length'] );
				$piece->setWidth( $item['width'] );
				$piece->setHeight( $item['height'] );
				$piece->setNonStandard( $item['nonStandard'] );
				$piece->setEuroReturn( $item['euroReturn'] );
				$pieces[] = $piece;

				$this->set_is_dr_pickup( $item['weight'], $item['length'], $item['width'], $item['height'] );
			}
		} else {
			$pieces[] = $this->get_piece();
		}
		$shipment_info = $this->get_shipment_info();

		if ( isset( $this->data['content'] ) ) {
			$content = $this->data['content'];
		} else {
			$content = Option::get_instance()->get_value( 'content' );
		}
		if ( isset( $this->data['comment'] ) ) {
			$comment = $this->data['comment'];
		} else {
			$comment = Option::get_instance()->get_value( 'comment' );
		}
		$reference = null;
		if ( isset( $this->data['reference'] ) ) {
			$reference = $this->data['reference'];
		} else {
			$reference = Option::get_instance()->get_value( 'reference' );
		}

		$shipment_data = array(
			'ship'          => $ship,
			'shipment_info' => $shipment_info,
			'pieces'        => $pieces,
			'content'       => $content,
			'comment'       => $comment,
			'reference'     => $reference,
		);

		$service_point_repository = new ShipmentRepository();
		try {
			$shipment = $service_point_repository->create_shipment( $shipment_data );
		} catch ( ExceptionInterface $e ) {
			throw new HelperException( $e, $e->getMessage() );
		}

		if ( 'EK' === $shipment_info->getServiceType() ) {
			$shipment_id = $shipment->getPackagesTrackingNumbers();
		} else {
			$shipment_id = $shipment->getShipmentNotificationNumber();
		}
		$dispatch_number = $shipment->getDispatchNotificationNumber();
		$label           = $shipment->getLabel();

		if ( ! empty( $this->order_id ) ) {
			update_post_meta( $this->order_id, 'dhl24_shipment_id', $shipment_id );
			update_post_meta( $this->order_id, 'dhl24_dispatch_number', $dispatch_number );
		}

		return array(
			'shipment_id'     => $shipment_id,
			'dispatch_number' => $dispatch_number,
			'mimeType'        => $label->getLabelFormat(),
			'fileData'        => $label->getLabelContent(),
		);
	}

	/**
	 * Get company address
	 *
	 * @return ShipperAddress
	 */
	private function get_shipper_address(): ShipperAddress {
		$data = Option::get_instance()->get_data();

		$address = new ShipperAddress();
		$address->setCountry( $data['country'] );
		$address->setName( $data['name'] );
		$address->setPostalCode( $this->formatter->post_code( $data['postal_code'], $data['country'] ) );
		$address->setCity( $data['city'] );
		$address->setStreet( $data['street'] );
		$address->setHouseNumber( $data['house_nmber'] );
		$address->setApartmentNumber( $data['apartment_number'] );

		return $address;
	}

	/**
	 * Get company contact
	 *
	 * @return PreavisoContact
	 */
	private function get_contact(): PreavisoContact {
		$data = Option::get_instance()->get_data();

		$contact = new PreavisoContact();
		$contact->setPersonName( $data['contact_person'] );
		$contact->setPhoneNumber( $data['contact_phone'] );
		$contact->setEmailAddress( $data['contact_email'] );

		return $contact;
	}

	/**
	 * Get company preaviso
	 *
	 * @return PreavisoContact
	 */
	private function get_preaviso(): PreavisoContact {
		$data = Option::get_instance()->get_data();

		$preaviso = new PreavisoContact();
		$preaviso->setPersonName( $data['contact_person'] );
		$preaviso->setPhoneNumber( $data['contact_phone'] );
		$preaviso->setEmailAddress( $data['contact_email'] );

		return $preaviso;
	}

	/**
	 * Get piece
	 *
	 * @return PieceDefinition
	 */
	private function get_piece(): PieceDefinition {
		$data                 = Option::get_instance()->get_data();
		$data['piece_weight'] = $this->normalizer->float( $data['piece_weight'] );

		$piece = new PieceDefinition();
		$piece->setType( $data['piece_type'] );
		$piece->setWidth( $data['piece_width'] );
		$piece->setHeight( $data['piece_height'] );
		$piece->setLength( $data['piece_length'] );
		$piece->setWeight( $data['piece_weight'] );
		$piece->setQuantity( $data['piece_quantity'] );

		$this->set_is_dr_pickup( $data['piece_weight'], $data['piece_length'], $data['piece_width'], $data['piece_height'] );

		return $piece;
	}

	/**
	 * Get piece
	 *
	 * @return ShipmentInfo
	 */
	private function get_shipment_info(): ShipmentInfo {
		$default_data = Option::get_instance()->get_data();

		if ( ! isset( $this->data['shipment_date'] ) ) {
			$this->data['shipment_date'] = null;
		}
		if ( ! isset( $this->data['costs_center'] ) ) {
			$this->data['costs_center'] = $default_data['costs_center'];
		}

		$shipment_info = new ShipmentInfo();
		$shipment_info->setServiceType( $this->data['shipping_method'] );
		if ( 'yes' === $default_data['auto_order_courier'] ) {
			$shipment_info->setDropOffType( 'REQUEST_COURIER' );
		} else {
			$shipment_info->setDropOffType( 'REGULAR_PICKUP' );
		}

		$billing = new Billing();
		$billing->setShippingPaymentType( 'SHIPPER' );
		$billing->setBillingAccountNumber( $default_data['sap'] );
		$billing->setPaymentType( 'BANK_TRANSFER' );
		$billing->setCostsCenter( $this->data['costs_center'] );
		$shipment_info->setBilling( $billing );

		$shipment_time = new ShipmentTime();
		if ( $this->data['shipment_date'] instanceof \DateTime ) {
			$shipment_time->setShipmentDate( $this->data['shipment_date'] );
		} else {
			$shipment_time->setShipmentDate( $this->services->get_pickup_day( $default_data['postal_code'], $this->is_dr_pickup ) );
		}
		$shipment_time->setShipmentStartHour( $default_data['shipment_start_hour'] );
		$shipment_time->setShipmentEndHour( $default_data['shipment_end_hour'] );
		$shipment_info->setShipmentTime( $shipment_time );

		$shipment_info->setLabelType( $default_data['label_type'] );

		if ( isset( $this->data['service'] ) && is_array( $this->data['service'] ) ) {
			if ( true === $this->data['service']['insurance'] ) {
				$service = new Service();
				$service->setServiceType( 'UBEZP' );
				$service->setServiceValue( $this->data['service']['insuranceValue'] );
				$shipment_info->addSpecialServices( $service );
			}
			if ( true === $this->data['service']['collectOnDelivery'] ) {
				$service = new Service();
				$service->setServiceType( 'COD' );
				$service->setServiceValue( $this->data['service']['collectOnDeliveryValue'] );
				$service->setCollectOnDeliveryForm( 'BANK_TRANSFER' );
				$shipment_info->addSpecialServices( $service );
			}
			if ( true === $this->data['service']['predeliveryInformation'] ) {
				$service = new Service();
				$service->setServiceType( 'PDI' );
				$shipment_info->addSpecialServices( $service );
			}
			if ( true === $this->data['service']['proofOfDelivery'] ) {
				$service = new Service();
				$service->setServiceType( 'POD' );
				$shipment_info->addSpecialServices( $service );
			}
			if ( true === $this->data['service']['returnOnDelivery'] ) {
				$service = new Service();
				$service->setServiceType( 'ROD' );
				$service->setTextInstruction( $this->data['service']['returnOnDeliveryReference'] );
				$shipment_info->addSpecialServices( $service );
			}
			if ( true === $this->data['service']['selfCollect'] ) {
				$service = new Service();
				$service->setServiceType( 'ODB' );
				$shipment_info->addSpecialServices( $service );
			}
			if ( true === $this->data['service']['deliveryOnSaturday'] ) {
				$service = new Service();
				$service->setServiceType( 'SOBOTA' );
				$shipment_info->addSpecialServices( $service );
			}
		}

		return $shipment_info;
	}

	/**
	 * Set is_dr_pickup
	 *
	 * @param  float|null $weight Weight.
	 * @param  int|null   $length Length.
	 * @param  int|null   $width Width.
	 * @param  int|null   $height Height.
	 * @return void
	 */
	private function set_is_dr_pickup( $weight, $length, $width, $height ) {
		if ( 31.5 < $weight ) {
			$this->is_dr_pickup = true;
		}
		if ( 50 > $weight && ( 200 < $length || 200 < $width || 200 < $height ) ) {
			$this->is_dr_pickup = true;
		}
	}
}
