<?php
/**
 * Class ServicePointRepository.
 *
 * @package ILabs\WoocommerceDHL24
 */

namespace ILabs\WoocommerceDHL24\Repository;

use ILabs\WoocommerceDHL24\Helper\Option;
use ILabs\WoocommerceDHL24\Helper\Formatter;
use ILabs\WoocommerceDHL24\Exception\RepositoryException;
use ILabs\WoocommerceDHL24\DHL24Client\ServicePoint\Type\Label;
use ILabs\WoocommerceDHL24\DHL24Client\ServicePoint\Type\AuthData;
use ILabs\WoocommerceDHL24\DHL24Client\ServicePoint\Type\Shipment;
use ILabs\WoocommerceDHL24\DHL24Client\ServicePoint\Method\GetLabel;
use ILabs\WoocommerceDHL24\DHL24Client\ServicePoint\ServicePointClient;
use ILabs\WoocommerceDHL24\DHL24Client\ServicePoint\Method\CreateShipment;
use ILabs\WoocommerceDHL24\DHL24Client\ServicePoint\Type\NearestServicepoints;
use ILabs\WoocommerceDHL24\DHL24Client\ServicePoint\Method\GetNearestServicepoints;

/**
 * ServicePointRepository
 */
class ServicePointRepository {
	/**
	 * ServicePointClient
	 *
	 * @var ServicePointClient
	 */
	protected $service_point_client;

	/**
	 * AuthData
	 *
	 * @var AuthData
	 */
	protected $auth;

	/**
	 * Formatter
	 *
	 * @var Formatter
	 */
	protected $formatter;

	/**
	 * Constructor.
	 */
	public function __construct() {
		$data                       = Option::get_instance()->get_data();
		$this->service_point_client = new ServicePointClient( $data['servicepoint_wsdl'] );
		$this->auth                 = new AuthData( $data['servicepoint_username'], $data['servicepoint_password'] );

		$this->formatter = new Formatter();
	}

	/**
	 * Fetch nearest service points
	 *
	 * @param  string $post_code Postal code.
	 * @param  int    $radius  Radius in km.
	 * @return NearestServicepoints
	 * @throws RepositoryException If error.
	 */
	public function fetch_nearest_service_points( string $post_code, int $radius = 10 ): NearestServicepoints {
		$post_code = $this->formatter->post_code( $post_code );

		$params = new GetNearestServicepoints( $this->auth );
		$params->setPostcode( $post_code );
		$params->setRadius( $radius );

		try {
			$nearest_service_points = $this->service_point_client->getNearestServicepoints( $params );
		} catch ( \Exception $e ) {
			throw new RepositoryException( $e, $e->getMessage() );
		}

		return $nearest_service_points;
	}

	/**
	 * Create shipments
	 *
	 * @param  array $shipment_data Array of shipment data.
	 * @return Shipment
	 * @throws RepositoryException If error.
	 */
	public function create_shipment( array $shipment_data ):Shipment {
		$params = new CreateShipment( $this->auth );
		$params->setShip( $shipment_data['ship'] );
		$params->setShipmentInfo( $shipment_data['shipment_info'] );
		foreach ( $shipment_data['pieces'] as $piece ) {
			$params->addPiece( $piece );
		}
		$params->setContent( $shipment_data['content'] );
		$params->setComment( $shipment_data['comment'] );
		$params->setReference( $shipment_data['reference'] );

		try {
			$shipment = $this->service_point_client->createShipment( $params );
		} catch ( \Exception $e ) {
			throw new RepositoryException( $e, $e->getMessage() );
		}

		return $shipment;
	}

	/**
	 * Fetch information on labels
	 *
	 * @param  string $shipment_id Shipment number of the shipment for which information is to be downloaded.
	 * @param  string $label_type Label type.
	 * @return Label
	 * @throws RepositoryException If error.
	 */
	public function fetch_label_print_out( string $shipment_id, string $label_type ): Label {
		$params = new GetLabel( $this->auth );
		$params->setShipment( $shipment_id );
		$params->setType( $label_type );

		try {
			$labels = $this->service_point_client->getLabel( $params );
		} catch ( \Exception $e ) {
			throw new RepositoryException( $e, $e->getMessage() );
		}

		return $labels;
	}
}
