<?php
/**
 * Class WooCommerceShippingDHL24.
 *
 * @package ILabs\WoocommerceDHL24
 */

declare(strict_types=1);
namespace ILabs\WoocommerceDHL24\ShippingMethod;

use WC_Shipping_Flat_Rate;

/**
 * WooCommerceShippingDHL24
 */
class WooCommerceShippingDHL24 extends WC_Shipping_Flat_Rate {

	/**
	 * Constructor.
	 *
	 * @param int $instance_id Shipping method instance ID.
	 */
	public function __construct( $instance_id = 0 ) {
		$this->instance_id = absint( $instance_id );
		$this->supports    = array(
			'shipping-zones',
			'instance-settings',
			'instance-settings-modal',
		);
		$this->init();

		add_action( 'woocommerce_update_options_shipping_' . $this->id, array( $this, 'process_admin_options' ) );
	}

	/**
	 * Init user set variables.
	 */
	public function init() {
		$this->init_form_fields();
		$this->title      = $this->get_option( 'title' );
		$this->tax_status = $this->get_option( 'tax_status' );
		$this->cost       = $this->get_option( 'cost' );
		$this->type       = $this->get_option( 'type', 'class' );
	}

	/**
	 * Init form fields.
	 */
	public function init_form_fields() {
		$cost_desc = __( 'Enter a cost (excl. tax) or sum, e.g. <code>10.00 * [qty]</code>.', 'woocommerce-dhl24' ) . '<br/><br/>' . __( 'Use <code>[qty]</code> for the number of items, <br/><code>[cost]</code> for the total cost of items, and <code>[fee percent="10" min_fee="20" max_fee=""]</code> for percentage based fees.', 'woocommerce-dhl24' );

		$this->instance_form_fields = array(
			'title'      => array(
				'title'    => __( 'Method title', 'woocommerce-dhl24' ),
				'type'     => 'text',
				'default'  => $this->method_title,
				'desc_tip' => true,
			),
			'tax_status' => array(
				'title'   => __( 'Tax status', 'woocommerce-dhl24' ),
				'type'    => 'select',
				'class'   => 'wc-enhanced-select',
				'default' => 'taxable',
				'options' => array(
					'taxable' => __( 'Taxable', 'woocommerce-dhl24' ),
					'none'    => _x( 'None', 'Tax status', 'woocommerce-dhl24' ),
				),
			),
			'cost'       => array(
				'title'             => __( 'Cost', 'woocommerce-dhl24' ),
				'type'              => 'text',
				'placeholder'       => '',
				'description'       => $cost_desc,
				'default'           => '0',
				'desc_tip'          => true,
				'sanitize_callback' => array( $this, 'sanitize_cost' ),
			),
		);

		$shipping_classes = WC()->shipping()->get_shipping_classes();

		if ( ! empty( $shipping_classes ) ) {
			$this->instance_form_fields['class_costs'] = array(
				'title'       => __( 'Shipping class costs', 'woocommerce-dhl24' ),
				'type'        => 'title',
				'default'     => '',
				/* translators: %s: URL for link. */
				'description' => sprintf( __( 'These costs can optionally be added based on the <a href="%s">product shipping class</a>.', 'woocommerce-dhl24' ), admin_url( 'admin.php?page=wc-settings&tab=shipping&section=classes' ) ),
			);
			foreach ( $shipping_classes as $shipping_class ) {
				if ( ! isset( $shipping_class->term_id ) ) {
					continue;
				}
				$this->instance_form_fields[ 'class_cost_' . $shipping_class->term_id ] = array(
					/* translators: %s: shipping class name */
					'title'             => sprintf( __( '"%s" shipping class cost', 'woocommerce-dhl24' ), esc_html( $shipping_class->name ) ),
					'type'              => 'text',
					'placeholder'       => __( 'N/A', 'woocommerce-dhl24' ),
					'description'       => $cost_desc,
					'default'           => $this->get_option( 'class_cost_' . $shipping_class->slug ), // Before 2.5.0, we used slug here which caused issues with long setting names.
					'desc_tip'          => true,
					'sanitize_callback' => array( $this, 'sanitize_cost' ),
				);
			}

			$this->instance_form_fields['no_class_cost'] = array(
				'title'             => __( 'No shipping class cost', 'woocommerce-dhl24' ),
				'type'              => 'text',
				'placeholder'       => __( 'N/A', 'woocommerce-dhl24' ),
				'description'       => $cost_desc,
				'default'           => '',
				'desc_tip'          => true,
				'sanitize_callback' => array( $this, 'sanitize_cost' ),
			);

			$this->instance_form_fields['type'] = array(
				'title'   => __( 'Calculation type', 'woocommerce-dhl24' ),
				'type'    => 'select',
				'class'   => 'wc-enhanced-select',
				'default' => 'class',
				'options' => array(
					'class' => __( 'Per class: Charge shipping for each shipping class individually', 'woocommerce-dhl24' ),
					'order' => __( 'Per order: Charge shipping for the most expensive shipping class', 'woocommerce-dhl24' ),
				),
			);
		}
	}
}
